<?php

namespace FiloBlu\Design\Block;

use Magento\Directory\Helper\Data;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Registry;
use Magento\Framework\UrlInterface;
use Magento\Framework\View\Element\Template;
use Magento\Framework\View\Element\Template\Context;
use Magento\Framework\View\Page\Config;

class BeforeJs extends Template
{
    const CONFIG_IMAGE_DIR_PREFIX = 'filoblu/lazyload/';
    /**
     * Helper
     *
     * @var \FiloBlu\Design\Helper\Data
     */
    public $_helper;
    public $registry;
    public $_directoryHelper;

    /**
     * BeforeJs constructor.
     * @param Context $context
     * @param \FiloBlu\Design\Helper\Data $helper
     * @param Registry $registry
     * @param Data $directoryHelper
     * @param array $data
     */
    public function __construct(
        Context $context,
        \FiloBlu\Design\Helper\Data $helper,
        Registry $registry,
        Data $directoryHelper,
        array $data = []
    ) {
        $this->_helper = $helper;
        $this->registry = $registry;
        $this->_directoryHelper = $directoryHelper;

        $om = ObjectManager::getInstance();
        /** @var Config $page */
        $page = $om->get(Config::class);

        // VideoJS
        if ($this->_helper->getConfig('js_library/videojs_4_12_7_enabled')) {
            $page->addPageAsset('FiloBlu_Design::js/videoJS/css/video-js-4.12.7.min.css');
        }

        //plyr
        if ($this->_helper->getConfig('js_library/plyr_2_0_11_enabled')) {
            $page->addPageAsset('FiloBlu_Design::js/plyr/css/plyr.css');
        }

        if ($this->_helper->getConfig('js_library/plyr_2_0_18_enabled')) {
            $page->addPageAsset('FiloBlu_Design::js/plyr/css/plyr_2.0.18.css');
        }

        if ($this->_helper->getConfig('js_library/plyr_3_0_2_enabled')) {
            $page->addPageAsset('FiloBlu_Design::js/plyr/css/plyr_3.0.2.css');
        }

        // Swiper
        if ($this->_helper->getConfig('js_library/swiper_3_3_1_enabled')) {
            $page->addPageAsset('FiloBlu_Design::js/Swiper/css/swiper.min-3.3.1.css');
        }

        if ($this->_helper->getConfig('js_library/swiper_4_0_6_enabled')) {
            $page->addPageAsset('FiloBlu_Design::js/Swiper/css/swiper.min-4.0.6.css');
        }

        if ($this->_helper->getConfig('js_library/swiper_5_2_1_enabled')) {
            $page->addPageAsset('FiloBlu_Design::js/Swiper/css/swiper.min-5.2.1.css');
        }

        if ($this->_helper->getConfig('js_library/plyr_3_5_6_enabled')) {
            $page->addPageAsset('FiloBlu_Design::js/plyr/css/plyr_3.5.6.css');
        }

        if ($this->_helper->getConfig('js_library/plyr_3_6_2_enabled')) {
            $page->addPageAsset('FiloBlu_Design::js/plyr/css/plyr_3.6.2.css');
        }

        // nanoScroller
        if ($this->_helper->getConfig('js_library/nanoScrollerJS_0_8_7_enabled')) {
            $page->addPageAsset('FiloBlu_Design::js/nanoScrollerJS/css/nanoscroller-0.8.7.css');
        }

        // intlTelInput
        if ($this->_helper->getConfig('js_library/intlTelInput_enabled')) {
            $page->addPageAsset('FiloBlu_Design::js/intlTelInput/css/intlTelInput.css');
        }

        //		$page->addPageAsset('Dfe_Markdown::main.css');

        parent::__construct($context, $data);
    }

    /**
     * @return string
     * @throws NoSuchEntityException
     */
    public function getBaseUrlConfigImage()
    {
        return $this->_storeManager->getStore()->getBaseUrl(UrlInterface::URL_TYPE_MEDIA) . self::CONFIG_IMAGE_DIR_PREFIX;
    }

    public function getDisplayBreakPoint($_breakpointConfig)
    {
        $strToReturn = '';
        foreach ($_breakpointConfig as $conf) {
            $strToReturn .= $conf['breakpointtype'] . ':"' . $conf['breakpointtype'] . '",';
        }
        return $strToReturn;
    }

    public function getMediaQueriesBreakPoint($_breakpointConfig)
    {
        $strToReturn = '';
        foreach ($_breakpointConfig as $conf) {
            $strToReturn .= $conf['breakpointtype'] . ':"' . $conf['breakpointmediaquery'] . '",';
        }
        return $strToReturn;
    }

    protected function getCacheLifetime()
    {
        return 3600;
    }
}
