<?php

namespace FiloBlu\Design\Helper;

use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Helper\Image;
use Magento\Catalog\Model\CategoryRepository;
use Magento\Catalog\Model\Product\Url;
use Magento\Catalog\Model\ProductRepository;
use Magento\Catalog\Model\ResourceModel\Product;
use Magento\Cms\Model\Template\FilterProvider;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\Exception\FileSystemException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Filesystem;
use Magento\Framework\Image\AdapterFactory;
use Magento\Framework\Math\Random;
use Magento\Framework\Stdlib\DateTime\TimezoneInterface;
use Magento\Framework\UrlInterface;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;

/**
 * Class Data
 * @package FiloBlu\Design\Helper
 */
class Data extends AbstractHelper
{
    /** @var string */
    const CONTENT_DIR_IMAGE = 'contentmanager/content/';

    protected $_mathRandom;
    protected $_storeManager;
    protected $_categoryRepository;
    protected $_productRepository;
    protected $_resourceModelProduct;
    protected $_filesystem;
    protected $_imageFactory;
    protected $_timezoneInterface;
    protected $_directory_list;
    protected $_filterProvider;
    protected $_productImageHelper;
    protected $configuredBreakingPonts;
    protected $_urlInterface;

    /**
     * Data constructor.
     * @param Context $context
     * @param StoreManagerInterface $storeManager
     * @param CategoryRepository $categoryRepository
     * @param ProductRepository $productRepository
     * @param Product $resourceModelProduct
     * @param Filesystem $filesystem
     * @param AdapterFactory $imageFactory
     * @param Random $mathRandom
     * @param TimezoneInterface $timezoneInterface
     * @param DirectoryList $directory_list
     * @param FilterProvider $filterProvider
     * @param Image $productImageHelper
     */
    public function __construct(
        Context $context,
        StoreManagerInterface $storeManager,
        CategoryRepository $categoryRepository,
        ProductRepository $productRepository,
        Product $resourceModelProduct,
        Filesystem $filesystem,
        AdapterFactory $imageFactory,
        Random $mathRandom,
        TimezoneInterface $timezoneInterface,
        DirectoryList $directory_list,
        FilterProvider $filterProvider,
        Image $productImageHelper
    ) {
        parent::__construct($context);
        $this->_storeManager = $storeManager;
        $this->_categoryRepository = $categoryRepository;
        $this->_productRepository = $productRepository;
        $this->_resourceModelProduct = $resourceModelProduct;
        $this->_filesystem = $filesystem;
        $this->_imageFactory = $imageFactory;
        $this->_mathRandom = $mathRandom;
        $this->_urlInterface = $context->getUrlBuilder();
        $this->_timezoneInterface = $timezoneInterface;
        $this->_directory_list = $directory_list;
        $this->_filterProvider = $filterProvider;
        $this->_productImageHelper = $productImageHelper;
    }

    /**
     * @return string
     */
    public function getContentDirImage()
    {
        return self::CONTENT_DIR_IMAGE;
    }

    /**
     * @param $imgPathDesktop
     * @param $imgPathTablet
     * @param $imgPathMobile
     * @param $resizeWidthDesktop
     * @param $resizeHeightDesktop
     * @param $resizeWidthTablet
     * @param $resizeHeightTablet
     * @param $resizeWidthMobile
     * @param $resizeHeightMobile
     * @param $dataForceLoad
     * @param $lazyloadContainer
     * @param $class
     * @param $alt
     * @param $idImage
     * @param $createBgImageDiv
     * @param null $dataMageInit
     * @return string
     * @throws NoSuchEntityException
     */
    public function getImgHtml(
        $imgPathDesktop,
        $imgPathTablet,
        $imgPathMobile,
        $resizeWidthDesktop,
        $resizeHeightDesktop,
        $resizeWidthTablet,
        $resizeHeightTablet,
        $resizeWidthMobile,
        $resizeHeightMobile,
        $dataForceLoad,
        $lazyloadContainer,
        $class,
        $alt,
        $idImage,
        $createBgImageDiv,
        $dataMageInit = null
    ) {
        if (!is_file($this->getPathMedia($imgPathTablet))) {
            $imgPathTablet = null;
        }

        if (!is_file($this->getPathMedia($imgPathMobile))) {
            $imgPathMobile = null;
        }

        if ($imgPathTablet == $imgPathDesktop) {
            $imgPathTablet = null;
        }

        if ($imgPathMobile == $imgPathDesktop) {
            $imgPathMobile = null;
        }

        // Desktop and default data
        $imgWidthDesktop = null;
        $imgHeightDesktop = null;
        $imgFullPathDesktop = $this->getPathMedia($imgPathDesktop);

        if ($lazyloadContainer && is_file($imgFullPathDesktop)) {
            list($imgWidthDesktop, $imgHeightDesktop) = getimagesize($imgFullPathDesktop);
        }

        $url = $this->getImageResized($imgPathDesktop, $resizeWidthDesktop, $resizeHeightDesktop);
        $desktopData = "data-src=\"{$url}\" data-original-width=\"{$imgWidthDesktop}\" data-original-height=\"{$imgHeightDesktop}\"";

        # Tablet data
        $tabletData = '';

        if (!$imgPathTablet && (($resizeWidthTablet && ($resizeWidthTablet != $resizeWidthDesktop)) || ($resizeHeightTablet && ($resizeHeightTablet != $resizeHeightDesktop)))) {
            $imgPathTablet = $imgPathDesktop;
        }

        $imgWidthTablet = null;
        $imgHeightTablet = null;
        if ($imgPathTablet) {
            $imgFullPathTablet = $this->getPathMedia($imgPathTablet);

            if ($lazyloadContainer && is_file($imgFullPathTablet)) {
                list($imgWidthTablet, $imgHeightTablet) = getimagesize($imgFullPathTablet);
            }

            $src = $this->getImageResized($imgPathTablet, $resizeWidthTablet, $resizeHeightTablet);
            $tabletData = "data-src-tablet=\"{$src}\" data-src-tablet_retina=\"{$src}\" ";
            $tabletData .= "data-original-width-tablet=\"{$imgWidthTablet}\" data-original-height-tablet=\"{$imgHeightTablet}\" ";
            $tabletData .= "data-original-width-tablet_retina=\"{$imgWidthTablet}\" data-original-height-tablet_retina=\"{$imgHeightTablet}\"";
        }

        # Mobile data
        $mobileData = '';

        if (!$imgPathMobile && (($resizeWidthMobile && ($resizeWidthMobile != $resizeWidthDesktop)) || ($resizeHeightMobile && ($resizeHeightMobile != $resizeHeightDesktop)))) {
            $imgPathMobile = $imgPathDesktop;
        }

        $imgWidthMobile = null;
        $imgHeightMobile = null;
        if ($imgPathMobile) {
            $imgFullPathMobile = $this->getPathMedia($imgPathMobile);

            if ($lazyloadContainer && is_file($imgFullPathMobile)) {
                list($imgWidthMobile, $imgHeightMobile) = getimagesize($imgFullPathMobile);
            }

            $dataSrcMobile = $this->getImageResized($imgPathMobile, $resizeWidthMobile, $resizeHeightMobile);
            $mobileData  = "data-src-mobile=\"{$dataSrcMobile}\" data-src-mobile_retina=\"{$dataSrcMobile}\" ";
            $mobileData .= "data-original-width-mobile=\"{$imgWidthMobile}\" data-original-height-mobile=\"{$imgHeightMobile}\" ";
            $mobileData .= "data-original-width-mobile_retina=\"{$imgWidthMobile}\" data-original-height-mobile_retina=\"{$imgHeightMobile}\"";
        }

        $img = '';
        $rndId = $this->getRandomString();

        if ($lazyloadContainer) {
            $img .= "<span class=\"lazyload-container\" id=\"lazyload-container_{$rndId}\">";
        }

        $dataForceLoadData = ($dataForceLoad ? 'data-force-load="true"' : '');
        $bgElementData = '';

        if ($createBgImageDiv) {
            if (!$dataForceLoadData) {
                $dataForceLoadData = 'data-force-load="true"';
            }

            $class .= ' no-display';
            $bgElementData = "data-bg-element-id=\"#{$rndId}\"";
            $loadingImage = $this->getLoadingImage();
            $img .= "<div class=\"lazyload bg-image\" id=\"{$rndId}\" style=\"background-image:url('{$loadingImage}')\"></div>";
        }

        if ($dataMageInit) {
            $dataMageInit = "data-mage-init='{$dataMageInit}'";
        }

        $src = $lazyloadContainer ? $this->getTransparentImage() : $this->getLoadingImage();
        $id = empty($idImage) ? '' : "id=\"{$idImage}\"";
        $img .= "<img {$id} src=\"{$src}\" {$bgElementData} {$desktopData} {$tabletData} {$mobileData} {$dataForceLoadData} {$dataMageInit} class=\"lazyload {$class}\" alt=\"{$alt}\" />";

        $_breakpointConfig = $this->getConfiguredBreakingPoints();
        $_mediaQueries = null;

        foreach ($_breakpointConfig as $conf) {
            if ($conf['breakpointtype'] === 'mobile' || ($conf['breakpointtype'] === 'mobile_retina' && $imgPathMobile)) {
                $paddingBottom = $this->calculateLazyLoadPaddingBottom($mobileData ? $imgWidthMobile : $imgWidthDesktop, $mobileData ? $imgHeightMobile : $imgHeightDesktop);
                $_mediaQueries .=
                    "@media only screen and {$conf['breakpointmediaquery']} {
                        #lazyload-container_{$rndId} {
                            padding-bottom: {$paddingBottom};
                        }
                    }";
            }

            if ($conf['breakpointtype'] === 'tablet' || ($conf['breakpointtype'] === 'tablet_retina' && $imgPathTablet)) {
                $paddingBottom = $this->calculateLazyLoadPaddingBottom($tabletData ? $imgWidthTablet : $imgWidthDesktop, $tabletData ? $imgHeightTablet : $imgHeightDesktop);
                $_mediaQueries .=
                    "@media only screen and {$conf['breakpointmediaquery']} {
                        #lazyload-container_{$rndId} {
                            padding-bottom: {$paddingBottom};
                        }
                    }";
            }

            if ($conf["breakpointtype"] === 'desktop') {
                $paddingBottom = $this->calculateLazyLoadPaddingBottom($imgWidthDesktop, $imgHeightDesktop);
                $_mediaQueries .=
                    "@media only screen and {$conf['breakpointmediaquery']} {
                        #lazyload-container_{$rndId} {
                            padding-bottom: {$paddingBottom};
                        }
                    }";
            }
        }

        if ($lazyloadContainer) {
            $img .= "</span><style>{$_mediaQueries}</style>";
        }

        return $img;
    }

    public function getPathMedia($path)
    {
        return $this->_filesystem->getDirectoryRead(DirectoryList::MEDIA)->getAbsolutePath($path);
    }

    /**
     * @param $image
     * @param null $width
     * @param null $height
     * @return string
     * @throws NoSuchEntityException
     * @throws \Exception
     */
    public function getImageResized($image, $width = null, $height = null)
    {
        if (!$width && !$height) {
            return $this->getUrlMedia($image);
        }

        $absolutePath = $this->getPathMedia($image);
        $destination = "contentmanager/content/resized/{$width}x{$height}/{$image}";

        if (is_file($this->getPathMedia($destination))) {
            return $this->getUrlMedia($destination);
        }

        if (is_file($absolutePath)) {
            $imageResize = $this->_imageFactory->create();
            $imageResize->open($absolutePath);
            $imageResize->constrainOnly(true);
            $imageResize->keepTransparency(true);
            $imageResize->keepFrame(false);
            $imageResize->keepAspectRatio(true);
            $imageResize->resize($width, $height);
            $imageResize->quality($this->getConfig('images_settings/image_quality_resize'));
            //save image
            $imageResize->save($this->getPathMedia($destination));

            return $this->getUrlMedia($destination);
        }

        return '';
    }

    /**
     * @param $path
     * @return string
     * @throws NoSuchEntityException
     */
    public function getUrlMedia($path)
    {
        return $this->_storeManager->getStore()->getBaseUrl(UrlInterface::URL_TYPE_MEDIA) . $path;
    }

    /**
     * @param $config_path
     * @param string $section
     * @return mixed
     */
    public function getConfig($config_path, $section = 'filoblu_design_section')
    {
        return $this->scopeConfig->getValue("{$section}/{$config_path}",ScopeInterface::SCOPE_STORE);
    }

    public function getRandomString()
    {
        return str_replace([' ', '.'], '',uniqid('f', true));
    }

    public function getLoadingImage()
    {
        try {
            return $this->getUrlMedia('filoblu/lazyload/' . $this->getConfig('js_library/lazyload_image_loading'));
        } catch (NoSuchEntityException $e) {
            return '';
        }
    }

    public function getTransparentImage()
    {
        try {
            return $this->getUrlMedia('filoblu/lazyload/' . $this->getConfig('js_library/lazyload_image_transparent'));
        } catch (NoSuchEntityException $e) {
            return '';
        }
    }

    public function getConfiguredBreakingPoints()
    {
        if (!$this->configuredBreakingPonts) {

            $this->configuredBreakingPonts = unserialize($this->getConfig('js_library/breakpoint_config'));

            if($this->configuredBreakingPonts === null) {
                $this->configuredBreakingPonts = [];
            }
        }

        return $this->configuredBreakingPonts;
    }

    /**
     * Return padding in percentage
     *
     * @param $width
     * @param $height
     * @return string
     */
    public function calculateLazyLoadPaddingBottom($width, $height)
    {
        if ($width && $height) {
            return round((($height / $width) * 100.00), 1) . '%';
        }

        return '';
    }

    /**
     * @param $productId
     * @return string|null
     */
    public function getProductUrlById($productId)
    {
        if ($productId) {
            try {
                $product = $this->_productRepository->getById($productId);
                /** @var Url $urlModel */
                $urlModel = $product->getUrlModel();
                return $urlModel->getUrl($product);
            } catch (NoSuchEntityException $e) {
                return null;
            }
        }

        return null;
    }

    /**
     * @param string $productSku
     * @return ProductInterface|mixed|null
     */
    public function getProductDataBySku($productSku)
    {
        try {
            return $this->_productRepository->get($productSku);
        } catch (NoSuchEntityException $e) {
            return null;
        }
    }

    public function getProductDataById($productId)
    {
        if ($productId) {
            try {
                return $this->_productRepository->getById($productId);
            } catch (NoSuchEntityException $e) {
                return null;
            }
        }

        return null;
    }

    /**
     * @param $productSku
     * @return ProductInterface|\Magento\Catalog\Model\Product|null
     */
    public function getProductInstanceBySku($productSku)
    {
        try {
            return $this->_productRepository->get($productSku);
        } catch (NoSuchEntityException $e) {
            return null;
        }
    }

    /**
     * @param $productId
     * @return ProductInterface|mixed|null
     */
    public function getProductInstanceById($productId)
    {
        if (!$productId) {
            return null;
        }

        try {
            return $this->_productRepository->getById($productId);
        } catch (NoSuchEntityException $e) {
            return null;
        }
    }

    public function cleanHtmlForSharing($text, $type = 'standard')
    {
        $encoding = mb_detect_encoding($text, 'UTF-8, ISO-8859-1');
        if ($type === 'standard') {
            $outp_chr = ['...', "'", "'", '"', '"', '•', '-', '-'];
        } elseif ($type === 'reference') {
            $outp_chr = ['&#8230;', '&#8216;', '&#8217;', '&#8220;', '&#8221;', '&#8226;', '&#8211;', '&#8212;']; // decimal numerical character references
        }
        $utf8_chr = ["\xe2\x80\xa6", "\xe2\x80\x98", "\xe2\x80\x99", "\xe2\x80\x9c", "\xe2\x80\x9d", '\xe2\x80\xa2', "\xe2\x80\x93", "\xe2\x80\x94"]; // UTF-8 hex characters
        $winc_chr = [chr(133), chr(145), chr(146), chr(147), chr(148), chr(149), chr(150), chr(151)]; // ASCII characters (found in Windows-1252)
        $text = str_replace([$utf8_chr, $winc_chr], [$outp_chr, $outp_chr], $text);
        $text = mb_convert_encoding($text, 'UTF-8', $encoding);

        $text = preg_replace('/\s+/', ' ', addslashes($text));

        return $text;
    }

    /**
     * @param $linkType
     * @param $linkCustom
     * @param $linkCategory
     * @param $linkProduct
     * @return string
     * @throws NoSuchEntityException
     */
    public function getContentUrl($linkType, $linkCustom, $linkCategory, $linkProduct)
    {
        $_url = null;

        switch ($linkType) {
            case 'default':
            case 'none':
                $_url = '';
                break;
            case 'custom':
                $_url = $this->getFilteredContent($linkCustom);
                break;

            case 'category':
                $_url = $this->getCategoryUrlById($linkCategory);
                break;

            case 'product':
                $_url = $this->getProductUrlBySku($linkProduct);
                break;
        }

        return trim($_url);
    }

    /**
     * @param $string
     * @return mixed
     * @throws NoSuchEntityException
     */
    public function getFilteredContent($string)
    {
        return $this->_filterProvider->getBlockFilter()->setStoreId($this->_storeManager->getStore()->getId())->filter($string);
    }

    /**
     * @param $categoryId
     * @return string|null
     */
    public function getCategoryUrlById($categoryId)
    {
        if (!$categoryId) {
            return null;
        }
        try {
            return $this->_categoryRepository->get($categoryId)->getUrl();
        } catch (NoSuchEntityException $e) {
            return null;
        }
    }

    /**
     * @param $productSku
     * @return string|null
     */
    public function getProductUrlBySku($productSku)
    {
        try {
            $product = $this->_productRepository->get($productSku);

            if ($product === null) {
                return null;
            }

            $urlModel = $product->getUrlModel();

            return $urlModel->getUrl($product);
        } catch (NoSuchEntityException $e) {
            return null;
        }
    }

    public function cleanContentManagerImages()
    {
        $this->deleteDirectory('contentmanager/content/resized/');
    }

    /**
     * @param string $path
     */
    public function deleteDirectory($path)
    {
        try {
            $this->_filesystem->getDirectoryWrite(DirectoryList::MEDIA)->delete($path);
        } catch (FileSystemException $e) {
            $this->_logger->warning("Unable to delete {$path}");
        }
    }

    /**
     * @param string $imgUrlDesktop
     * @param int $imgOriginalWidth
     * @param int $imgOriginalHeight
     * @param bool $dataForceLoad
     * @param bool $lazyloadContainer
     * @param string $class
     * @param $alt
     * @param $idImage
     * @param null $dataMageInit
     * @param null $lowQualityPreview
     * @return string
     * @throws NoSuchEntityException
     */
    public function getImgListingHtml(
        $imgUrlDesktop,
        $imgOriginalWidth,
        $imgOriginalHeight,
        $dataForceLoad,
        $lazyloadContainer,
        $class,
        $alt,
        $idImage,
        $dataMageInit = null,
        $lowQualityPreview = null
    ) {
        $type = '';
        $_product = null;
        $width = '';
        $height = '';
        $lowQualityUrl = '';
        $rndId = $this->getRandomString();

        /* if $lowQuality Preview */
        if ($lowQualityPreview && is_array($lowQualityPreview)) {
            if (array_key_exists('type', $lowQualityPreview)) {
                $type = $lowQualityPreview['type'];
            }
            if (array_key_exists('product', $lowQualityPreview)) {
                $_product = $lowQualityPreview['product'];
            }
            if (array_key_exists('width', $lowQualityPreview) && array_key_exists('height', $lowQualityPreview)) {
                $width = $lowQualityPreview['width'];
                $height = $lowQualityPreview['height'];
            } elseif (array_key_exists('width', $lowQualityPreview) && !array_key_exists('height', $lowQualityPreview)) {
                $width = $lowQualityPreview['width'];
                $height = $this->calculateLowQualityImageProportion($imgOriginalWidth, $imgOriginalHeight, $lowQualityPreview['width'], 'width');
            } elseif (array_key_exists('height', $lowQualityPreview) && !array_key_exists('width', $lowQualityPreview)) {
                $height = $lowQualityPreview['height'];
                $width = $this->calculateLowQualityImageProportion($imgOriginalWidth, $imgOriginalHeight, $lowQualityPreview['height'], 'height');
            }

            if ($type && $_product) {
                $lowQualityUrl = $this->_productImageHelper->init($_product, $type)
                    ->constrainOnly(false)
                    ->keepAspectRatio(true)
                    ->keepTransparency(true)
                    ->keepFrame(true)
                    ->resize($width, $height)->getUrl();
            }
        }

        if ($lazyloadContainer && (!$imgOriginalWidth || !$imgOriginalHeight)) {
            $baseUrl = $this->_storeManager->getStore()->getBaseUrl(UrlInterface::URL_TYPE_MEDIA);
            $imgFullPathDesktop = $this->getPathMedia(str_replace($baseUrl, '', $imgUrlDesktop));

            if (is_file($imgFullPathDesktop)) {
                list($imgOriginalWidth,$imgOriginalHeight) = getimagesize($imgFullPathDesktop);
            }

            if (!$imgOriginalWidth || !$imgOriginalHeight) {
                $lazyloadContainer = false;
            }
        }

        $desktopData = "data-src=\"{$imgUrlDesktop}\" data-original-width=\"{$imgOriginalWidth}\" data-original-height=\"{$imgOriginalHeight}\"";

        $img = '';

        if ($lazyloadContainer && !$lowQualityPreview) {
            $img .= "<span id=\"lazyload-container_{$rndId}\" class=\"lazyload-container\">";
        } elseif ($lazyloadContainer && $lowQualityPreview) {
            $paddingBottom = $this->calculateLazyLoadPaddingBottom($imgOriginalWidth, $imgOriginalHeight);
            $img .= "<span class=\"lazyload-low-quality lazyload-container\" style=\"padding-bottom: {$paddingBottom};background-image:url('{$lowQualityUrl}')\">";
        }

        $dataForceLoadData = ($dataForceLoad ? 'data-force-load="true"' : '');

        if ($dataMageInit) {
            $dataMageInit = "data-mage-init='{$dataMageInit}'";
        }

        $src = ($lazyloadContainer ? $this->getTransparentImage() : $this->getLoadingImage());
        $id = empty($idImage) ? '' : "id=\"{$idImage}\"";
        $img .= "<img {$id} src=\"{$src}\" {$desktopData} {$dataForceLoadData} {$dataMageInit} class=\"lazyload {$class}\" alt=\"{$alt}\" />";

        if ($lazyloadContainer) {
            $paddingBottom = $this->calculateLazyLoadPaddingBottom($imgOriginalWidth, $imgOriginalHeight);
            $img .= "</span><style>#lazyload-container_{$rndId} { padding-bottom: $paddingBottom;}</style>";
        }

        return $img;
    }

    /**
     * @param int $originalWidth
     * @param int $originalHeight
     * @param float $contraint
     * @param string $c
     * @return false|float|string
     */
    public function calculateLowQualityImageProportion($originalWidth, $originalHeight, $contraint, $c = 'width')
    {
        $prop = '';
        if ($originalWidth && $originalHeight && $contraint && $c) {
            switch ($c) {
                case 'width':
                    // $originalWidth : $originalHeight = $contraint : x
                    $prop = ceil((($contraint * $originalHeight) / $originalWidth));
                    break;
                case 'height':
                    // $originalWidth : $originalHeight = x : $contraint
                    $prop = ceil((($contraint * $originalWidth) / $originalHeight));
                    break;
            }
            return $prop;
        }

        // TODO: provide a fallback value ?
        return '';
    }
}
